//go:generate go mod vendor
//nolint:lll
//go:generate protoc --go_out=plugins=grpc,paths=source_relative:. -I .:vendor/gitlab.mashroom.com/mashroom/backend mashroom_contract.proto
//go:generate rm -r vendor
//go:generate go fmt ./...
//go:generate go fix ./...

// Package contract provides contract service api
package contract // import "gitlab.mashroom.com/mashroom/backend/contract/api"

import (
	"context"
	"os"

	// generic proto
	_ "gitlab.mashroom.com/mashroom/backend/common/api/generic"
	_ "gitlab.mashroom.com/mashroom/backend/common/api/protobuf"
)

// AssetFunc bindata asset loading signature
type AssetFunc func(name string) ([]byte, error)

// AssetInfoFunc bindata asset metadata signature
type AssetInfoFunc func(name string) (os.FileInfo, error)

// TemplateID returns template ID for given asset, updated if neccessery
func TemplateID(
	ctx context.Context,
	assetFunc AssetFunc,
	infoFunc AssetInfoFunc,
	client ContractServiceClient,
	name, file string,
) (cid string, err error) {
	info, err := infoFunc(file)
	if err != nil {
		return "", err
	}

	list, err := client.List(ctx, &ListRequest{
		ParentId: "0",
	})
	if err != nil {
		return "", err
	}

	var tmpl *ContractInfo

	for _, c := range list.Children {
		if c.Name == name {
			tmpl = c
			cid = c.Id

			break
		}
	}

	create := false

	if tmpl == nil {
		var cr *CreateResponse

		cr, err = client.Create(ctx, &CreateRequest{
			ParentId: "0",
			Name:     name,
		})
		if err != nil {
			return "", err
		}

		tmpl = cr.GetInfo()
		cid = tmpl.Id
		create = true
	}

	if create || tmpl.TimeUpdated < info.ModTime().Unix() {
		var data []byte

		data, err = assetFunc(file)
		if err != nil {
			return "", err
		}

		var eresp *EditResponse

		eresp, err = client.Edit(ctx, &EditRequest{
			Id:   cid,
			Text: string(data),
		})
		if err != nil {
			return "", err
		}

		cid = eresp.GetInfo().GetId()
	}

	return cid, nil
}
