package main

import (
	"fmt"
	"net"
	"os"

	"github.com/jmoiron/sqlx"
	"gitlab.mashroom.com/mashroom/backend/common/generic/addr"
	"gitlab.mashroom.com/mashroom/backend/common/generic/env"
	"gitlab.mashroom.com/mashroom/backend/common/generic/grpc"
	logger "gitlab.mashroom.com/mashroom/backend/common/generic/log"
	contract "gitlab.mashroom.com/mashroom/backend/contract/api"
	"gitlab.mashroom.com/mashroom/backend/contract/internal/model/postgres"
	"gitlab.mashroom.com/mashroom/backend/contract/internal/service"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
)

var documentService = document.NewDocumentServiceClient(grpc.Dial(
	env.ResolveString("DOCUMENT_SERVICE_URL", "mashroom-document:10000"),
	"Document",
))

var envWeaverServiceURL = env.ResolveString("WEAVER_SERVICE", "http://127.0.0.1:8080/convert?auth=arachnys-weaver")

func main() {
	conn := fmt.Sprintf(
		"postgres://%s:%s@%s:%s/%s?sslmode=disable",
		os.Getenv("POSTGRES_USERNAME"),
		os.Getenv("POSTGRES_PASSWORD"),
		os.Getenv("POSTGRES_HOST"),
		os.Getenv("POSTGRES_PORT"),
		os.Getenv("POSTGRES_DATABASE"),
	)

	db := sqlx.MustOpen("postgres", conn)

	repo, err := postgres.NewRepository(db)
	if err != nil {
		panic(err)
	}

	connections := service.Connections{
		Document: documentService,
	}

	srv, err := service.NewService(&service.Config{
		Repo:        repo,
		Connections: connections,
		WeaverURL:   envWeaverServiceURL,
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("contract")
	if err != nil {
		panic(err)
	}

	contract.RegisterContractServiceServer(grpcServer, srv)

	logger.Infoln("contract started")

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
