// Package model provides structures for persistent repository
package model

//go:generate mockgen -package=mock -source=api.go -destination=mock/repository.go Repository

import "context"

// Repository interface abstracting specific repository
type Repository interface {
	Migrate() error
	ContractGet(ctx context.Context, id uint64) (*Contract, error)
	ContractCreate(ctx context.Context, contract *Contract) error
	ContractUpdate(ctx context.Context, contract *Contract) error
	ContractDelete(ctx context.Context, id uint64, force bool) error
	ContractRevision(ctx context.Context, id uint64, text string) (*Contract, error)
	ContractList(ctx context.Context, parentID uint64, deleted bool) ([]*Contract, error)
	RevisionList(ctx context.Context, contractID uint64) ([]*Revision, error)
	RevisionLoad(ctx context.Context, id uint64) (*Revision, error)
}
