// Package service provides implementation for contract service
package service

// nolint:lll
//go:generate mockgen -package=mock -destination=mock/mock_document_service_client.go gitlab.mashroom.com/mashroom/backend/document/api DocumentServiceClient
//go:generate mockgen -package=mock -destination=mock/mock_document_service_file_upload_client.go gitlab.mashroom.com/mashroom/backend/document/api DocumentService_FileUploadClient

import (
	"fmt"

	contract "gitlab.mashroom.com/mashroom/backend/contract/api"
	"gitlab.mashroom.com/mashroom/backend/contract/internal/model"
	document "gitlab.mashroom.com/mashroom/backend/document/api"
)

// Service contract
type Service interface {
	contract.ContractServiceServer
}

// Connections remote service connections
type Connections struct {
	Document document.DocumentServiceClient
}

// Config for Service
type Config struct {
	Repo        model.Repository
	Connections Connections
	WeaverURL   string
}

func validateConfigConnections(connections *Connections) error {
	if connections.Document == nil {
		return fmt.Errorf("no document client")
	}

	return nil
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	if err := validateConfigConnections(&config.Connections); err != nil {
		return err
	}

	if config.Repo == nil {
		return fmt.Errorf("no repository in service config")
	}

	if config.WeaverURL == "" {
		return fmt.Errorf("no weaver URL in service config")
	}

	return nil
}

// NewService returns new Service instance
func NewService(config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	return &serverImpl{
		Config: *config,
	}, nil
}

type serverImpl struct {
	Config
}
