package service

import (
	"net/http"
	"net/http/httptest"
	"testing"

	"github.com/golang/mock/gomock"

	"gitlab.mashroom.com/mashroom/backend/contract/internal/model/mock"
	cism "gitlab.mashroom.com/mashroom/backend/contract/internal/service/mock"
)

type testDependencies struct {
	ctrl *gomock.Controller

	srv          Service
	repo         *mock.MockRepository
	docCli       *cism.MockDocumentServiceClient
	docUploadCli *cism.MockDocumentService_FileUploadClient
	serverMock   *httptest.Server
}

func newDependencies(t *testing.T, weaverHandler http.HandlerFunc) *testDependencies {
	ctrl := gomock.NewController(t)

	repo := mock.NewMockRepository(ctrl)

	docCli := cism.NewMockDocumentServiceClient(ctrl)

	docUploadCli := cism.NewMockDocumentService_FileUploadClient(ctrl)

	servMock := cism.ServerMock(weaverHandler)

	srv := serverImpl{
		Config: Config{
			Repo: repo,
			Connections: Connections{
				Document: docCli,
			},
			WeaverURL: servMock.URL,
		},
	}

	return &testDependencies{
		ctrl: ctrl,

		srv:          &srv,
		repo:         repo,
		docCli:       docCli,
		docUploadCli: docUploadCli,
		serverMock:   servMock,
	}
}
